#!/bin/bash
set -o errexit -o pipefail -o noclobber -o nounset

DEPENDENCY_TO_BUILD="all"
while [ $# -gt 0 ]; do
    case "$1" in
        --coyote-root)
            coyote_ROOT=$2
            ABSOLUTE_coyote_ROOT=$(readlink -f "${coyote_ROOT}")
            shift 2
            ;;
        --dependency)
            DEPENDENCY_TO_BUILD=$2
            shift 2
            ;;
        *)
            echo "Error: unknown argument '$1'. Call this script with --coyote-root and path to your coyote-root"
            exit 1
            ;;
    esac
done

echo "Absolute path provided as the coyote root: $ABSOLUTE_coyote_ROOT"

COMPILER="mpich-gcc-13"
export CC="mpicc"
export FC="mpif90"
export CXX="mpicxx"
export MPI_LAUNCH="mpiexec"

THREADS=8

BUILD_PATH=$(pwd)/$COMPILER
INSTALL_PATH=$BUILD_PATH/install
mkdir -p "$BUILD_PATH"
pushd "$BUILD_PATH"

# we try to keep things as close to ICON installation as possible
if $(uname -s | grep -q "Darwin"); then
    # Assume that the packages are installed to '/opt/local'
    # (default for MacPorts):
    COYOTE_SW_PREFIX="/opt/local"
else
    COYOTE_SW_PREFIX="/usr"
fi

COYOTE_SW_PREFIX="$INSTALL_PATH;$COYOTE_SW_PREFIX"
echo "Dependencies will be checked in these paths: $COYOTE_SW_PREFIX"

# set the python interpretor to the local one
VENV_PATH=$BUILD_PATH/py_venv
python3.11 -m venv $VENV_PATH
source $VENV_PATH/bin/activate

# build dependencies for coyote
echo "=== Building coyote dependencies ==="
CC="${CC}" CXX="${CXX}" FC="${FC}" $ABSOLUTE_coyote_ROOT/scripts/setup_devenv/build_dependencies.sh -j $THREADS --dependency $DEPENDENCY_TO_BUILD --prefix_path $INSTALL_PATH

# build coyote
echo "=== Building coyote ==="
CC="${CC}" CXX="${CXX}" FC="${FC}" cmake $ABSOLUTE_coyote_ROOT -DCMAKE_PREFIX_PATH=$COYOTE_SW_PREFIX -DCMAKE_BUILD_TYPE=Debug
cmake --build . -j $THREADS
popd


pip install -r ${ABSOLUTE_coyote_ROOT}/requirements-dev.txt
cp $BUILD_PATH/python/coyote.*.so $VENV_PATH/lib/python3.11/site-packages/

export PYTHONPATH=${BUILD_PATH}/python:${ABSOLUTE_coyote_ROOT}/apps
echo $PYTHONPATH


if python -c "import coyote; from hiopy._data_handler import DataHandler" &> /dev/null ; then
    echo "Coyote libraries installed and usable successfully"
else
    echo "Something went wrong in appropriate installation of coyote and hiopy"
fi
